#include <Arduino.h>
#include <WiFi.h>
#include <string.h>

#include "Dictionary.h"
#include "TLV.h"
#include "WiFiMgr.h"

#define MAX_WAIT	10000
#define CONN_WAIT	250


bool
SetupWiFi()
{
	WiFi.mode(WIFI_STA);
	return true;
}


static bool
tryConnect(const char *ssid, const char *wpakey)
{
	size_t		 waitedFor = 0;

	Serial.print("CONNECT ");
	Serial.println(ssid);
	WiFi.begin(ssid, wpakey);
	while ((WiFi.status() != WL_CONNECTED) && (waitedFor < MAX_WAIT)) {
		waitedFor += CONN_WAIT;
		if ((waitedFor % 1000) == 0) {
			Serial.print(".");
		}
		delay(250);
	}

	if (WiFi.status() == WL_CONNECTED) {
		Serial.print("MODEM ADDR ");
		Serial.println(WiFi.localIP());
		return true;
	}

	Serial.println("CARRIER SIGNAL LOST");
	return false;
}


bool
TryConnect(Dictionary &pb)
{
	TLV::Record	 ssid;
	TLV::Record	 wpakey;
	uint8_t		*cursor;

	while ((cursor != NULL) && (cursor[0] != TAG_EMPTY)) {
		TLV::ReadFromMemory(ssid, cursor);
		TLV::SkipRecord(ssid, cursor);
		
		TLV::ReadFromMemory(wpakey, cursor);
		TLV::SkipRecord(ssid, cursor);

		if (tryConnect(ssid.Val, wpakey.Val)) {
			return true;
		}
	}

	return false;
}


static bool
tryConnect(Dictionary &pb, int network)
{
	const char	*ssid = WiFi.SSID(network).c_str();
	size_t		 ssidLen = strnlen(ssid, TLV_MAX_LEN);
	TLV::Record	 password;

	if (ssidLen == 0) {
		return false;
	}

	Serial.print("MODEM: CHECK ");
	Serial.println(ssid);

	if (!pb.Lookup(ssid, uint8_t(ssidLen), password)) {
		Serial.println("SSID NOT IN PHONEBOOK");
		return false;
	}

	return tryConnect(ssid, password.Val);
}

bool
Autoconnect(Dictionary &pb)
{
	return Autoconnect(pb, true);
}


bool
Autoconnect(Dictionary &pb, bool reset)
{
	int	networkCount = 0;
	int	network = 0;

	Serial.println("MODEM: TRY AUTOCONNECT");

	if (reset) {
		Serial.println("RADIO RESET");
		SetupWiFi();
		WiFi.disconnect();
		delay(1000);
	} else {
		Serial.println("NO RESET");
	}

	networkCount = WiFi.scanNetworks();
	for (network = 0; network < networkCount; network++) {
		if (tryConnect(pb, network)) {
			return true;
		}
	}

	Serial.println("NO CARRIER");
	return false;
}

